import { cassandra } from "@lib/cassandra";
import { jsonResponse } from "@lib/http";
import { pika } from "@lib/pika";
import {
	isValidEmail,
	isValidPassword,
	isValidUsername,
} from "@lib/validators";

const routeDef: RouteDef = {
	method: "POST",
	accepts: "application/json",
	returns: "application/json;charset=utf-8",
	needsBody: "json",
};

async function handler(
	_request: ExtendedRequest,
	requestBody: unknown,
): Promise<Response> {
	const { username, password, email } = requestBody as {
		username?: string;
		password?: string;
		email?: string;
	};

	const fields = { username, password, email };

	const validators = {
		username: isValidUsername,
		password: isValidPassword,
		email: isValidEmail,
	};

	const errors: string[] = [];

	for (const [key, validate] of Object.entries(validators)) {
		const value = fields[key as keyof typeof fields];
		if (typeof value !== "string" || value.trim() === "") {
			errors.push(`${key} is required`);
			continue;
		}

		const result = validate(value);
		if (!result.valid) errors.push(`${key}: ${result.error}`);
	}

	if (errors.length > 0) {
		return jsonResponse(400, {
			message: "Validation failed",
			error: errors.join("; "),
		});
	}

	const usernameResult = await cassandra
		.getClient()
		.execute("SELECT id FROM users WHERE username = ?", [username], {
			prepare: true,
		});

	const emailResult = await cassandra
		.getClient()
		.execute("SELECT id FROM users WHERE email = ?", [email], {
			prepare: true,
		});

	if (usernameResult.rowLength > 0 || emailResult.rowLength > 0) {
		const errorMessages: string[] = [];

		if (usernameResult.rowLength > 0) {
			errorMessages.push("Username has already been taken");
		}
		if (emailResult.rowLength > 0) {
			errorMessages.push("Email has already been taken");
		}

		return jsonResponse(400, {
			message: "Validation failed",
			error: errorMessages.join("; "),
		});
	}

	const userId = pika.gen("user");

	return jsonResponse(200, {
		message: "User registered successfully",
		data: {
			username,
			email,
			id: userId,
		},
	});
}

export { handler, routeDef };
